local _, S = ...
local pairs, ipairs, string, type, time = pairs, ipairs, string, type, time

-- Default currency sort
local function DefaultSort(entry1, entry2)
    return entry1.name < entry2.name
end


-- CURRENCY GROUPS
S.currencyGroupingSettings = {
    ["selectedGrouping"] = "CATEGORY",
    ["collapsedGroups"] = {}
}
S.CurrencyGroups = {
    ["CATEGORY"] = {
        ["name"] = S.Localize("GROUPING_CATEGORY"),
        ["func"] = function(currency)
            return currency.heading, currency.heading
        end
    },
    ["QUALITY"] = {
        ["name"] = S.Localize("FILTER_QUALITY"),
        ["func"] = function(currency)
            if currency.quality then
                return _G["ITEM_QUALITY"..currency.quality.."_DESC"], -currency.quality
            else
                return "None", 100
            end
        end
    },
    ["FAVORITES"] = {
        ["name"] = S.Localize("FILTER_MARKER_ICON"),
        ["func"] = function(currency)
            local markerIcon = S.Data.GetCurrencyFavorited(currency)
            if not markerIcon then
                markerIcon = 100
            end
            return S.Utils.FormatMarkerIcon(markerIcon), markerIcon
        end
    }
}


-- Currency Columns
local function GetColumnWidth(self)
    return self.width
end
S.CurrencyColumns = {
    ["FAVORITES"] = {
        ["name"] = S.Localize("COLUMN_FAVORITES"),
        ["width"] = 24,
        ["GetWidth"] = GetColumnWidth,
        ["align"] = "CENTER"
    },
    ["QUANTITY"] = {
        ["name"] = S.Localize("COLUMN_QUANTITY"),
        ["width"] = 42,
        ["GetWidth"] = GetColumnWidth,
        ["align"] = "RIGHT",
        ["sortMethods"] = {
            {
                ["title"] = "#",
                ["func"] = function(asc, slot1, slot2)
                    return S.Sort.ByKey(not asc, slot1, slot2, "count", DefaultSort)
                end
            }
        }
    },
    ["ICON"] = {
        ["name"] = S.Localize("COLUMN_ICON"),
        ["GetWidth"] = function() 
            return S.Settings.Get("iconSize") + S.Settings.Get("padding") * 2
        end,
        ["align"] = "CENTER",
        ["sortMethods"] = {
            {
                ["title"] = "",
                ["func"] = function(asc, slot1, slot2)
                    return S.Sort.ByKey(not asc, slot1, slot2, "quality", DefaultSort)
                end
            }
        }
    },
    ["NAME"] = {
        ["name"] = S.Localize("COLUMN_NAME"),
        ["GetWidth"] = function(self) return nil end,
        ["align"] = "CENTER",
        ["sortMethods"] = {
            {
                ["title"] = S.Localize("COLUMN_RARITY_SHORT"),
                ["func"] = function(asc, slot1, slot2) 
                    return S.Sort.ByKey(not asc, slot1, slot2, "quality", DefaultSort)
                end
            },
            {
                ["title"] = S.Localize("COLUMN_NAME_SHORT"),
                ["func"] = function(asc, slot1, slot2) 
                    return S.Sort.ByKey(asc, slot1, slot2, "name", DefaultSort)
                end,
                ["inverse"] = true
            }
        }
    },
    ["CATEGORY"] = {
        ["name"] = S.Localize("GROUPING_CATEGORY"),
        ["width"] = 120,
        ["GetWidth"] = GetColumnWidth,
        ["align"] = "LEFT",
        ["sortMethods"] = {
            {
                ["title"] = S.Localize("GROUPING_CATEGORY"),
                ["func"] = function(asc, slot1, slot2) 
                    return S.Sort.ByKey(asc, slot1, slot2, "heading", DefaultSort)
                end,
                ["inverse"] = true
            }
        }
    },
    ["MAX-QUANTITY"] = {
        ["name"] = S.Localize("COLUMN_MAX_QUANTITY"),
        ["width"] = 42,
        ["GetWidth"] = GetColumnWidth,
        ["align"] = "LEFT",
        ["sortMethods"] = {
            {
                ["title"] = S.Localize("COLUMN_MAX_QUANTITY"),
                ["func"] = function(asc, slot1, slot2)
                    return S.Sort.ByKey(not asc, slot1, slot2, "maxQuantity", DefaultSort)
                end
            }
        }
    },
    ["WEEKLY-QUANTITY"] = {
        ["name"] = S.Localize("COLUMN_WEEKLY_QUANTITY"),
        ["width"] = 60,
        ["GetWidth"] = GetColumnWidth,
        ["align"] = "RIGHT",
        ["sortMethods"] = {
            {
                ["title"] = S.Localize("COLUMN_WEEKLY_QUANTITY_SHORT"),
                ["func"] = function(asc, slot1, slot2)
                    return S.Sort.ByKey(not asc, slot1, slot2, "quantityEarnedThisWeek", DefaultSort)
                end
            }
        }
    },
    ["MAX-WEEKLY-QUANTITY"] = {
        ["name"] = S.Localize("COLUMN_MAX_WEEKLY_QUANTITY"),
        ["width"] = 60,
        ["GetWidth"] = GetColumnWidth,
        ["align"] = "LEFT",
        ["sortMethods"] = {
            {
                ["title"] = S.Localize("COLUMN_MAX_WEEKLY_QUANTITY_SHORT"),
                ["func"] = function(asc, slot1, slot2)
                    return S.Sort.ByKey(not asc, slot1, slot2, "maxWeeklyQuantity", DefaultSort)
                end
            }
        }
    }
}


local function UpdateEntryDataTable(self)
    self.entryData = {}
    local data = S.GetData()
    if data.currencies then
        for k,v in pairs(data.currencies) do
            table.insert(self.entryData, {
                ["currencyID"] = k
            })
        end
    end
end

local function EntryHasData(self, entry)
    local data = self:GetDataForEntry(entry)
    return data and data.count > 0
end
local function GetDataForEntry(self, entry)
    local data = S.GetData()
    if data.currencies and entry then
        return data.currencies[entry.currencyID]
    end
end



local function GetEntryFavorited(self, entryData)
    return S.Data.GetCurrencyFavorited(entryData)
end



function S.CreateCurrencyList(parent)
    local self =  S.CreateList(parent, S.CreateCurrencyEntry, S.CurrencyColumns, "currencyColumnSettings", true, S.CurrencyGroups, S.currencyGroupingSettings)
    self.AddCurrencies = AddCurrencies
    self.EntryHasData = EntryHasData
    self.GetDataForEntry = GetDataForEntry
    self.UpdateEntryDataTable = UpdateEntryDataTable
    self.GetEntryFavorited = GetEntryFavorited
    
    S.Utils.RunOnEvent(self, "EnteredWorld", UpdateEntryDataTable)
    S.Utils.RunOnEvent(self, "CurrenciesUpdated", function(self)
        if self:IsShown() then
            self:UpdateEntryDataTable()
            self:ScheduleUpdate(false, true)
        end
    end)
    S.Utils.RunOnEvent(self, "CharacterSelected", function(self)
        self.columnSettings = S.Settings.Get(self.columnSettingsKey)
        if self:IsShown() then
            self:UpdateEntryDataTable()
            self:ScheduleUpdate(false, true)
        end
    end)

    -- Update everything on show, since updates are disabled when the frame is hidden for performance
    self:SetScript("OnShow", function(self)
        self:UpdateEntryDataTable()
        self:OnShow()
    end)

    return self
end